@@ -28,6 +28,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 
 #include <glib.h>
 
@@ -68,7 +69,7 @@ static void at_gprs_set_attached(struct ofono_gprs *gprs, int attached,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	snprintf(buf, sizeof(buf), "AT+CGATT=%i", attached ? 1 : 0);
@@ -78,8 +79,7 @@ static void at_gprs_set_attached(struct ofono_gprs *gprs, int attached,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -115,12 +115,20 @@ static void at_gprs_registration_status(struct ofono_gprs *gprs,
 	struct gprs_data *gd = ofono_gprs_get_data(gprs);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = gd;
 
 	switch (gd->vendor) {
+	case OFONO_VENDOR_GOBI:
+		/*
+		 * Send *CNTI=0 to find out the current tech, it will be
+		 * intercepted in gobi_cnti_notify in network registration
+		 */
+		g_at_chat_send(gd->chat, "AT*CNTI=0", none_prefix,
+				NULL, NULL, NULL);
+		break;
 	case OFONO_VENDOR_NOVATEL:
 		/*
 		 * Send $CNTI=0 to find out the current tech, it will be
@@ -136,8 +144,7 @@ static void at_gprs_registration_status(struct ofono_gprs *gprs,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, data);
 }
@@ -176,6 +183,51 @@ static void cgev_notify(GAtResult *result, gpointer user_data)
 	}
 }
 
+static void xdatastat_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_gprs *gprs = user_data;
+	GAtResultIter iter;
+	int stat;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+XDATASTAT:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &stat))
+
+	DBG("stat %d", stat);
+
+	switch (stat) {
+	case 0:
+		ofono_gprs_suspend_notify(gprs, GPRS_SUSPENDED_UNKNOWN_CAUSE);
+		break;
+	case 1:
+		ofono_gprs_resume_notify(gprs);
+		break;
+	}
+}
+
+static void cpsb_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_gprs *gprs = user_data;
+	GAtResultIter iter;
+	gint bearer;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CPSB:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, NULL))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &bearer))
+		return;
+
+	ofono_gprs_bearer_notify(gprs, bearer);
+}
+
 static void gprs_initialized(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_gprs *gprs = user_data;
@@ -183,7 +235,20 @@ static void gprs_initialized(gboolean ok, GAtResult *result, gpointer user_data)
 
 	g_at_chat_register(gd->chat, "+CGEV:", cgev_notify, FALSE, gprs, NULL);
 	g_at_chat_register(gd->chat, "+CGREG:", cgreg_notify,
-				FALSE, gprs, NULL);
+						FALSE, gprs, NULL);
+	g_at_chat_register(gd->chat, "+CPSB:", cpsb_notify, FALSE, gprs, NULL);
+
+	g_at_chat_send(gd->chat, "AT+CPSB=1", none_prefix, NULL, NULL, NULL);
+
+	switch (gd->vendor) {
+	case OFONO_VENDOR_IFX:
+		/* Register for GPRS suspend notifications */
+		g_at_chat_register(gd->chat, "+XDATASTAT:", xdatastat_notify,
+						FALSE, gprs, NULL);
+		g_at_chat_send(gd->chat, "AT+XDATASTAT=1", none_prefix,
+						NULL, NULL, NULL);
+		break;
+	}
 
 	ofono_gprs_register(gprs);
 }
@@ -229,13 +294,23 @@ static void at_cgreg_test_cb(gboolean ok, GAtResult *result,
 	g_at_chat_send(gd->chat, cmd, none_prefix, NULL, NULL, NULL);
 	g_at_chat_send(gd->chat, "AT+CGAUTO=0", none_prefix, NULL, NULL, NULL);
 
-	/* ST-E modem does not support AT+CGEREP = 2,1 */
-	if (gd->vendor == OFONO_VENDOR_STE)
+	switch (gd->vendor) {
+	case OFONO_VENDOR_MBM:
+		/* Ericsson MBM and ST-E modems don't support AT+CGEREP=2,1 */
 		g_at_chat_send(gd->chat, "AT+CGEREP=1,0", none_prefix,
 			gprs_initialized, gprs, NULL);
-	else
+		break;
+	case OFONO_VENDOR_NOKIA:
+		/* Nokia data cards don't support AT+CGEREP=1,0 either */
+		g_at_chat_send(gd->chat, "AT+CGEREP=1", none_prefix,
+			gprs_initialized, gprs, NULL);
+		break;
+	default:
 		g_at_chat_send(gd->chat, "AT+CGEREP=2,1", none_prefix,
 			gprs_initialized, gprs, NULL);
+		break;
+	}
+
 	return;
 
 error:
@@ -305,13 +380,16 @@ static int at_gprs_probe(struct ofono_gprs *gprs,
 	GAtChat *chat = data;
 	struct gprs_data *gd;
 
-	gd = g_new0(struct gprs_data, 1);
-	gd->chat = chat;
+	gd = g_try_new0(struct gprs_data, 1);
+	if (gd == NULL)
+		return -ENOMEM;
+
+	gd->chat = g_at_chat_clone(chat);
 	gd->vendor = vendor;
 
 	ofono_gprs_set_data(gprs, gd);
 
-	g_at_chat_send(chat, "AT+CGDCONT=?", cgdcont_prefix,
+	g_at_chat_send(gd->chat, "AT+CGDCONT=?", cgdcont_prefix,
 			at_cgdcont_test_cb, gprs, NULL);
 
 	return 0;
@@ -322,6 +400,8 @@ static void at_gprs_remove(struct ofono_gprs *gprs)
 	struct gprs_data *gd = ofono_gprs_get_data(gprs);
 
 	ofono_gprs_set_data(gprs, NULL);
+
+	g_at_chat_unref(gd->chat);
 	g_free(gd);
 }
 
@@ -333,12 +413,12 @@ static struct ofono_gprs_driver driver = {
 	.attached_status	= at_gprs_registration_status,
 };
 
-void at_gprs_init()
+void at_gprs_init(void)
 {
 	ofono_gprs_driver_register(&driver);
 }
 
-void at_gprs_exit()
+void at_gprs_exit(void)
 {
 	ofono_gprs_driver_unregister(&driver);
 }
