@@ -38,6 +38,7 @@
 #include "gatchat.h"
 #include "gatresult.h"
 
+#include "common.h"
 #include "atmodem.h"
 #include "vendor.h"
 
@@ -56,6 +57,7 @@ struct netreg_data {
 	int signal_min; /* min strength reported via CIND */
 	int signal_max; /* max strength reported via CIND */
 	int tech;
+	struct ofono_network_time time;
 	unsigned int vendor;
 };
 
@@ -81,7 +83,7 @@ static int option_parse_tech(GAtResult *result)
 {
 	GAtResultIter iter;
 	int s, octi, ouwcti;
-	int tech = -1;
+	int tech;
 
 	g_at_result_iter_init(&iter, result);
 
@@ -105,28 +107,31 @@ static int option_parse_tech(GAtResult *result)
 
 	switch (octi) {
 	case 1: /* GSM */
-		tech = 0;
+		tech = ACCESS_TECHNOLOGY_GSM;
 		break;
 	case 2: /* GPRS */
-		tech = 1;
+		tech = ACCESS_TECHNOLOGY_GSM;
 		break;
 	case 3: /* EDGE */
-		tech = 3;
+		tech = ACCESS_TECHNOLOGY_GSM_EGPRS;
+		break;
+	default:
+		tech = -1;
 		break;
 	}
 
 	switch (ouwcti) {
 	case 1: /* UMTS */
-		tech = 2;
+		tech = ACCESS_TECHNOLOGY_UTRAN;
 		break;
 	case 2: /* HSDPA */
-		tech = 4;
+		tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA;
 		break;
 	case 3: /* HSUPA */
-		tech = 5;
+		tech = ACCESS_TECHNOLOGY_UTRAN_HSUPA;
 		break;
 	case 4: /* HSPA */
-		tech = 6;
+		tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA_HSUPA;
 		break;
 	}
 
@@ -181,7 +186,7 @@ static void at_registration_status(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = nd;
@@ -195,6 +200,14 @@ static void at_registration_status(struct ofono_netreg *netreg,
 		g_at_chat_send(nd->chat, "AT*ERINFO?", none_prefix,
 				NULL, NULL, NULL);
 		break;
+	case OFONO_VENDOR_GOBI:
+		/*
+		 * Send *CNTI=0 to find out the current tech, it will be
+		 * intercepted in gobi_cnti_notify
+		 */
+		g_at_chat_send(nd->chat, "AT*CNTI=0", none_prefix,
+				NULL, NULL, NULL);
+		break;
 	case OFONO_VENDOR_NOVATEL:
 		/*
 		 * Send $CNTI=0 to find out the current tech, it will be
@@ -220,8 +233,7 @@ static void at_registration_status(struct ofono_netreg *netreg,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, -1, -1, -1, data);
 }
@@ -259,7 +271,7 @@ static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	/* Default to GSM */
 	if (g_at_result_iter_next_number(&iter, &tech) == FALSE)
-		tech = 0;
+		tech = ACCESS_TECHNOLOGY_GSM;
 
 	strncpy(op.name, name, OFONO_MAX_OPERATOR_NAME_LENGTH);
 	op.name[OFONO_MAX_OPERATOR_NAME_LENGTH] = '\0';
@@ -349,24 +361,35 @@ static void at_current_operator(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
 
-	ok = g_at_chat_send(nd->chat, "AT+COPS=3,2", none_prefix,
-				NULL, NULL, NULL);
+	/* Nokia modems have a broken return value for the string
+	 * returned for the numeric value. It misses a " at the end.
+	 * Trying to read this will stall the parser. So skip it. */
+	if (nd->vendor == OFONO_VENDOR_NOKIA) {
+		ok = g_at_chat_send(nd->chat, "AT+COPS=3,0", none_prefix,
+							NULL, NULL, NULL);
 
-	if (ok)
-		ok = g_at_chat_send(nd->chat, "AT+COPS?", cops_prefix,
-					cops_numeric_cb, cbd, NULL);
+		if (ok)
+			ok = g_at_chat_send(nd->chat, "AT+COPS?", cops_prefix,
+							cops_cb, cbd, NULL);
+	} else {
+		ok = g_at_chat_send(nd->chat, "AT+COPS=3,2", none_prefix,
+							NULL, NULL, NULL);
+
+		if (ok)
+			ok = g_at_chat_send(nd->chat, "AT+COPS?", cops_prefix,
+						cops_numeric_cb, cbd, NULL);
+	}
 
 	if (ok)
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, NULL, data);
 }
@@ -397,8 +420,7 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	DBG("Got %d elements", num);
 
 	list = g_try_new0(struct ofono_network_operator, num);
-
-	if (!list) {
+	if (list == NULL) {
 		CALLBACK_WITH_FAILURE(cb, 0, NULL, cbd->data);
 		return;
 	}
@@ -407,7 +429,7 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	g_at_result_iter_init(&iter, result);
 
 	while (g_at_result_iter_next(&iter, "+COPS:")) {
-		int status, tech;
+		int status, tech, plmn;
 		const char *l, *s, *n;
 		gboolean have_long = FALSE;
 
@@ -444,10 +466,13 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			extract_mcc_mnc(n, list[num].mcc, list[num].mnc);
 
 			if (!g_at_result_iter_next_number(&iter, &tech))
-				tech = 0;
+				tech = ACCESS_TECHNOLOGY_GSM;
 
 			list[num].tech = tech;
 
+			if (!g_at_result_iter_next_number(&iter, &plmn))
+				plmn = 0;
+
 			if (!g_at_result_iter_close_list(&iter))
 				break;
 
@@ -478,7 +503,7 @@ static void at_list_operators(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=?", cops_prefix,
@@ -486,8 +511,7 @@ static void at_list_operators(struct ofono_netreg *netreg,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, 0, NULL, data);
 }
@@ -509,7 +533,7 @@ static void at_register_auto(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=0", none_prefix,
@@ -517,8 +541,7 @@ static void at_register_auto(struct ofono_netreg *netreg,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -531,7 +554,7 @@ static void at_register_manual(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[128];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	snprintf(buf, sizeof(buf), "AT+COPS=1,2,\"%s%s\"", mcc, mnc);
@@ -541,28 +564,7 @@ static void at_register_manual(struct ofono_netreg *netreg,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
-
-	CALLBACK_WITH_FAILURE(cb, data);
-}
-
-static void at_deregister(struct ofono_netreg *netreg,
-				ofono_netreg_register_cb_t cb, void *data)
-{
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
-	struct cb_data *cbd = cb_data_new(cb, data);
-
-	if (!cbd)
-		goto error;
-
-	if (g_at_chat_send(nd->chat, "AT+COPS=2", none_prefix,
-				register_cb, cbd, g_free) > 0)
-		return;
-
-error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -621,6 +623,47 @@ static void option_osigq_notify(GAtResult *result, gpointer user_data)
 				at_util_convert_signal_strength(strength));
 }
 
+static void ifx_xhomezr_notify(GAtResult *result, gpointer user_data)
+{
+	//struct ofono_netreg *netreg = user_data;
+	const char *label;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+XHOMEZR:"))
+		return;
+
+	if (!g_at_result_iter_next_string(&iter, &label))
+		return;
+
+	ofono_info("Home zone: %s", label);
+}
+
+static void ifx_xciev_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	int strength, ind;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+XCIEV:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &ind))
+		return;
+
+	if (ind == 0)
+		strength = -1;
+	else if (ind == 7)
+		strength = 100;
+	else
+		strength = (ind * 15);
+
+	ofono_netreg_strength_notify(netreg, strength);
+}
+
 static void ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -646,6 +689,83 @@ static void ciev_notify(GAtResult *result, gpointer user_data)
 	ofono_netreg_strength_notify(netreg, strength);
 }
 
+static void ctzv_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	const char *tz;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CTZV:"))
+		return;
+
+	if (!g_at_result_iter_next_unquoted_string(&iter, &tz))
+		return;
+
+	DBG("tz %s", tz);
+
+	nd->time.utcoff = atoi(tz) * 15 * 60;
+
+	ofono_netreg_time_notify(netreg, &nd->time);
+}
+
+static void ifx_ctzv_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	int year, mon, mday, hour, min, sec;
+	const char *tz, *time;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CTZV:"))
+		return;
+
+	if (!g_at_result_iter_next_unquoted_string(&iter, &tz))
+		return;
+
+	if (!g_at_result_iter_next_string(&iter, &time))
+		return;
+
+	DBG("tz %s time %s", tz, time);
+
+	if (sscanf(time, "%u/%u/%u,%u:%u:%u", &year, &mon, &mday,
+						&hour, &min, &sec) != 6)
+		return;
+
+	nd->time.sec = sec;
+	nd->time.min = min;
+	nd->time.hour = hour;
+	nd->time.mday = mday;
+	nd->time.mon = mon;
+	nd->time.year = 2000 + year;
+}
+
+static void ifx_ctzdst_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	int dst;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CTZDST:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &dst))
+		return;
+
+	DBG("dst %d", dst);
+
+	nd->time.dst = dst;
+
+	ofono_netreg_time_notify(netreg, &nd->time);
+}
+
 static void cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -738,7 +858,7 @@ static void at_signal_strength(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = nd;
@@ -758,12 +878,60 @@ static void at_signal_strength(struct ofono_netreg *netreg,
 	}
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, data);
 }
 
+static void mbm_etzv_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	int year, mon, mday, hour, min, sec;
+	const char *tz, *time, *timestamp;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "*ETZV:") == FALSE)
+		return;
+
+	if (g_at_result_iter_next_string(&iter, &tz) == FALSE)
+		return;
+
+	if (g_at_result_iter_next_string(&iter, &time) == FALSE)
+		time = NULL;
+
+	if (g_at_result_iter_next_string(&iter, &timestamp) == FALSE)
+		timestamp = NULL;
+
+	DBG("tz %s time %s timestamp %s", tz, time, timestamp);
+
+	if (time == NULL) {
+		year = -1;
+		mon = -1;
+		mday = -1;
+		hour = -1;
+		min = -1;
+		sec = -1;
+	} else {
+		if (sscanf(time, "%u/%u/%u,%u:%u:%u", &year, &mon, &mday,
+						&hour, &min, &sec) != 6)
+		return;
+	}
+
+	nd->time.utcoff = atoi(tz) * 15 * 60;
+
+	nd->time.sec = sec;
+	nd->time.min = min;
+	nd->time.hour = hour;
+	nd->time.mday = mday;
+	nd->time.mon = mon;
+	nd->time.year = year;
+
+	ofono_netreg_time_notify(netreg, &nd->time);
+}
+
 static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -797,10 +965,10 @@ static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 	/* Convert to tech values from 27.007 */
 	switch (gsm) {
 	case 1: /* GSM */
-		nd->tech = 0;
+		nd->tech = ACCESS_TECHNOLOGY_GSM;
 		break;
 	case 2: /* EDGE */
-		nd->tech = 3;
+		nd->tech = ACCESS_TECHNOLOGY_GSM_EGPRS;
 		break;
 	default:
 		nd->tech = -1;
@@ -808,20 +976,60 @@ static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 
 	switch (umts) {
 	case 1: /* UMTS */
-		nd->tech = 2;
+		nd->tech = ACCESS_TECHNOLOGY_UTRAN;
 		break;
 	case 2: /* UMTS + HSDPA */
-		nd->tech = 4;
-		break;
-	default:
+		nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA;
 		break;
 	}
 }
 
+static int cnti_to_tech(const char *cnti)
+{
+	if (g_str_equal(cnti, "GSM") == TRUE ||
+			g_str_equal(cnti, "GPRS") == TRUE)
+		return ACCESS_TECHNOLOGY_GSM;
+	else if (g_str_equal(cnti, "EDGE") == TRUE)
+		return ACCESS_TECHNOLOGY_GSM_EGPRS;
+	else if (g_str_equal(cnti, "UMTS") == TRUE)
+		return ACCESS_TECHNOLOGY_UTRAN;
+	else if (g_str_equal(cnti, "HSDPA") == TRUE)
+		return ACCESS_TECHNOLOGY_UTRAN_HSDPA;
+	else if (g_str_equal(cnti, "HSUPA") == TRUE)
+		return ACCESS_TECHNOLOGY_UTRAN_HSUPA;
+
+	return -1;
+}
+
+static void gobi_cnti_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	const char *tech;
+	int option;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "*CNTI:") == FALSE)
+		return;
+
+	if (g_at_result_iter_next_number(&iter, &option) == FALSE)
+		return;
+
+	if (option != 0)
+		return;
+
+	if (g_at_result_iter_next_unquoted_string(&iter, &tech) == FALSE)
+		return;
+
+	nd->tech = cnti_to_tech(tech);
+}
+
 static void nw_cnti_notify(GAtResult *result, gpointer user_data)
 {
-	//struct ofono_netreg *netreg = user_data;
-	//struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *tech;
 	int option;
@@ -840,17 +1048,29 @@ static void nw_cnti_notify(GAtResult *result, gpointer user_data)
 	if (g_at_result_iter_next_unquoted_string(&iter, &tech) == FALSE)
 		return;
 
-	ofono_info("CNTI: %s", tech);
+	nd->tech = cnti_to_tech(tech);
 }
 
-static void option_query_tech_cb(gboolean ok,
-			GAtResult *result, gpointer user_data)
+static void cnti_query_tech_cb(gboolean ok, GAtResult *result,
+						gpointer user_data)
 {
 	struct tech_query *tq = user_data;
-	int tech = -1;
+	struct netreg_data *nd = ofono_netreg_get_data(tq->netreg);
+
+	ofono_netreg_status_notify(tq->netreg,
+			tq->status, tq->lac, tq->ci, nd->tech);
+}
+
+static void option_query_tech_cb(gboolean ok, GAtResult *result,
+						gpointer user_data)
+{
+	struct tech_query *tq = user_data;
+	int tech;
 
 	if (ok)
 		tech = option_parse_tech(result);
+	else
+		tech = -1;
 
 	ofono_netreg_status_notify(tq->netreg,
 			tq->status, tq->lac, tq->ci, tech);
@@ -870,26 +1090,36 @@ static void creg_notify(GAtResult *result, gpointer user_data)
 	if (status != 1 && status != 5)
 		goto notify;
 
+	tq = g_try_new0(struct tech_query, 1);
+	if (tq == NULL)
+		goto notify;
+
+	tq->status = status;
+	tq->lac = lac;
+	tq->ci = ci;
+	tq->netreg = netreg;
+
 	switch (nd->vendor) {
+	case OFONO_VENDOR_GOBI:
+		if (g_at_chat_send(nd->chat, "AT*CNTI=0", none_prefix,
+					cnti_query_tech_cb, tq, g_free) > 0)
+			return;
+		break;
+	case OFONO_VENDOR_NOVATEL:
+		if (g_at_chat_send(nd->chat, "AT$CNTI=0", none_prefix,
+					cnti_query_tech_cb, tq, g_free) > 0)
+			return;
+		break;
 	case OFONO_VENDOR_OPTION_HSO:
-		tq = g_new0(struct tech_query, 1);
-		if (!tq)
-			break;
-
-		tq->status = status;
-		tq->lac = lac;
-		tq->ci = ci;
-		tq->netreg = netreg;
-
 		if (g_at_chat_send(nd->chat, "AT_OCTI?;_OUWCTI?",
 					option_tech_prefix,
 					option_query_tech_cb, tq, g_free) > 0)
 			return;
-
-		g_free(tq);
 		break;
 	}
 
+	g_free(tq);
+
 	if ((status == 1 || status == 5) && tech == -1)
 		tech = nd->tech;
 
@@ -961,7 +1191,6 @@ error:
 	ofono_netreg_remove(netreg);
 }
 
-
 static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -996,15 +1225,46 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 				NULL, NULL, NULL);
 		g_at_chat_send(nd->chat, "AT_OSQI?", none_prefix,
 				NULL, NULL, NULL);
+
+		/* Register for network time update reports */
+		g_at_chat_register(nd->chat, "+CTZV:", ctzv_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+CTZR=1", none_prefix,
+						NULL, NULL, NULL);
 		break;
 	case OFONO_VENDOR_MBM:
+		/* Enable network registration updates */
+		g_at_chat_send(nd->chat, "AT*E2REG=1", none_prefix,
+						NULL, NULL, NULL);
+		g_at_chat_send(nd->chat, "AT*EREG=2", none_prefix,
+						NULL, NULL, NULL);
+		g_at_chat_send(nd->chat, "AT*EPSB=1", none_prefix,
+						NULL, NULL, NULL);
+
+		/* Register for network technology updates */
 		g_at_chat_send(nd->chat, "AT*ERINFO=1", none_prefix,
-				NULL, NULL, NULL);
+						NULL, NULL, NULL);
 		g_at_chat_register(nd->chat, "*ERINFO:", mbm_erinfo_notify,
-					FALSE, netreg, NULL);
+						FALSE, netreg, NULL);
+
+		/* Register for network time update reports */
+		g_at_chat_register(nd->chat, "*ETZV:", mbm_etzv_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT*ETZR=2", none_prefix,
+						NULL, NULL, NULL);
+
 		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
-				cind_support_cb, netreg, NULL);
+					cind_support_cb, netreg, NULL);
 		return;
+	case OFONO_VENDOR_GOBI:
+		/*
+		 * Gobi devices don't support unsolicited notifications
+		 * of technology changes, but register a handle for
+		 * CNTI so we get notified by any query.
+		 */
+		g_at_chat_register(nd->chat, "*CNTI:", gobi_cnti_notify,
+					FALSE, netreg, NULL);
+		break;
 	case OFONO_VENDOR_NOVATEL:
 		/*
 		 * Novatel doesn't support unsolicited notifications
@@ -1018,6 +1278,31 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_register(nd->chat, "^RSSI:", huawei_rssi_notify,
 					FALSE, netreg, NULL);
 		break;
+	case OFONO_VENDOR_IFX:
+		/* Register for specific signal strength reports */
+		g_at_chat_register(nd->chat, "+XCIEV:", ifx_xciev_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+XMER=1", none_prefix,
+						NULL, NULL, NULL);
+
+		/* Register for home zone reports */
+		g_at_chat_register(nd->chat, "+XHOMEZR:", ifx_xhomezr_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+XHOMEZR=1", none_prefix,
+						NULL, NULL, NULL);
+
+		/* Register for network time update reports */
+		g_at_chat_register(nd->chat, "+CTZV:", ifx_ctzv_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_register(nd->chat, "+CTZDST:", ifx_ctzdst_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+CTZR=1", none_prefix,
+						NULL, NULL, NULL);
+		break;
+	case OFONO_VENDOR_ZTE:
+	case OFONO_VENDOR_NOKIA:
+		/* Signal strength reporting via CIND is not supported */
+		break;
 	default:
 		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
 				cind_support_cb, netreg, NULL);
@@ -1083,12 +1368,20 @@ static int at_netreg_probe(struct ofono_netreg *netreg, unsigned int vendor,
 
 	nd = g_new0(struct netreg_data, 1);
 
-	nd->chat = chat;
+	nd->chat = g_at_chat_clone(chat);
 	nd->vendor = vendor;
 	nd->tech = -1;
+	nd->time.sec = -1;
+	nd->time.min = -1;
+	nd->time.hour = -1;
+	nd->time.mday = -1;
+	nd->time.mon = -1;
+	nd->time.year = -1;
+	nd->time.dst = 0;
+	nd->time.utcoff = 0;
 	ofono_netreg_set_data(netreg, nd);
 
-	g_at_chat_send(chat, "AT+CREG=?", creg_prefix,
+	g_at_chat_send(nd->chat, "AT+CREG=?", creg_prefix,
 			at_creg_test_cb, netreg, NULL);
 
 	return 0;
@@ -1100,6 +1393,7 @@ static void at_netreg_remove(struct ofono_netreg *netreg)
 
 	ofono_netreg_set_data(netreg, NULL);
 
+	g_at_chat_unref(nd->chat);
 	g_free(nd);
 }
 
@@ -1112,16 +1406,15 @@ static struct ofono_netreg_driver driver = {
 	.list_operators			= at_list_operators,
 	.register_auto			= at_register_auto,
 	.register_manual		= at_register_manual,
-	.deregister			= at_deregister,
 	.strength			= at_signal_strength,
 };
 
-void at_netreg_init()
+void at_netreg_init(void)
 {
 	ofono_netreg_driver_register(&driver);
 }
 
-void at_netreg_exit()
+void at_netreg_exit(void)
 {
 	ofono_netreg_driver_unregister(&driver);
 }
