@@ -236,7 +236,7 @@ static void hfp_registration_status(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
@@ -247,8 +247,7 @@ static void hfp_registration_status(struct ofono_netreg *netreg,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, -1, -1, -1, data);
 }
@@ -260,7 +259,7 @@ static void hfp_current_operator(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
@@ -285,7 +284,7 @@ static void hfp_signal_strength(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
@@ -295,8 +294,7 @@ static void hfp_signal_strength(struct ofono_netreg *netreg,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, data);
 }
@@ -350,12 +348,12 @@ static struct ofono_netreg_driver driver = {
 	.strength			= hfp_signal_strength,
 };
 
-void hfp_netreg_init()
+void hfp_netreg_init(void)
 {
 	ofono_netreg_driver_register(&driver);
 }
 
-void hfp_netreg_exit()
+void hfp_netreg_exit(void)
 {
 	ofono_netreg_driver_unregister(&driver);
 }
