@@ -355,8 +355,8 @@ out:
 
 static void hfp_dial(struct ofono_voicecall *vc,
 			const struct ofono_phone_number *ph,
-			enum ofono_clir_option clir, enum ofono_cug_option cug,
-			ofono_voicecall_cb_t cb, void *data)
+			enum ofono_clir_option clir, ofono_voicecall_cb_t cb,
+			void *data)
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	struct cb_data *cbd = cb_data_new(cb, data);
@@ -590,9 +590,8 @@ static void hfp_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 	req->data = data;
 	req->affected_types = 0;
 
-	/* strlen("AT+VTS=") = 7 */
-	buf = g_try_new(char, strlen(dtmf) + 7);
-
+	/* strlen("AT+VTS=) = 7 + NULL */
+	buf = g_try_new(char, strlen(dtmf) + 8);
 	if (buf == NULL)
 		goto error;
 
@@ -1174,12 +1173,12 @@ static struct ofono_voicecall_driver driver = {
 	.send_tones		= hfp_send_dtmf
 };
 
-void hfp_voicecall_init()
+void hfp_voicecall_init(void)
 {
 	ofono_voicecall_driver_register(&driver);
 }
 
-void hfp_voicecall_exit()
+void hfp_voicecall_exit(void)
 {
 	ofono_voicecall_driver_unregister(&driver);
 }
